@@ -34,19 +34,28 @@
 #' s['axesLayout'] <- 'parallel'
 #' states <- l_info_states(s)
 #' names(states)
+
+
 l_serialaxes <- function(data, sequence, scaling="variable", axesLayout='radial', 
-                         showAxes=TRUE, parent=NULL, ... ) {
+                         showAxes=TRUE, parent=NULL, ... ){
     
 
-    if (!missing(data) && missing(sequence)) {
+    data <- as.data.frame(data)
+    
+    if (missing(sequence)) {
         sequence <- names(data)
     }
     
-    loonPlotFactory('::loon::serialaxes', 'serialaxes', 'loon serialaxes plot', parent,
+    plot <- loonPlotFactory('::loon::serialaxes', 'serialaxes', 'loon serialaxes plot', parent,
                     data=l_data(data),
                     sequence=sequence,
                     showAxes=showAxes,
                     scaling=scaling,
                     axesLayout=axesLayout,
                     ...)
+    
+    class(plot) <- c("l_serialaxes", class(plot))
+    
+    plot
+
 }
\ No newline at end of file
