@@ -1,8 +1,6 @@
 
 #' Create and optionally draw a grid grob from a loon widget handle
 #'
-#' Grid grobs are useful to create publication quality graphics.
-#'
 #' @template param_target
 #'
 #' @template param_gridname
@@ -13,13 +11,13 @@
 #'
 #' @template param_gridvp
 #'
-#' @return a grid grob
+#' @return a grid grob of the loon plot
 #'
 #' @import grid
 #' @import grDevices
 #' @import stats
 #'
-#' @seealso \code{\link{loonGrob}}
+#' @seealso \code{\link{loonGrob}}, \code{\link{plot.loon}}
 #'
 #' @examples
 #'
@@ -38,6 +36,40 @@ grid.loon <- function (target, name = NULL, gp = gpar(), draw = TRUE, vp = NULL)
     invisible(lg)
 }
 
+
+
+#' Plot the current view of any loon plot in the current device.
+#'
+#' This is a wrapper for \code{grid.loon()} to simplify the plotting of
+#' loon plots on any device.  Frequent users are recommended to use
+#' \code{grid.loon()} for more control.
+#'
+#' @param x the loon plot to be plotted on the current device
+#' @param y NULL, will be ignored.
+#' @param ... parameters passed to \code{loonGrob}
+#'
+#' @return invisible()
+#'
+#'
+#' @seealso \code{\link{loonGrob}}, \code{\link{grid.loon}}
+#'
+#' @examples
+#'
+#' loonPlot <- with(iris, l_plot(Sepal.Length, Sepal.Width))
+#' loonPlot['color'] <- iris$Species
+#' loonPlot['selected'] <- iris$Species == "versicolor"
+#' l_scaleto_selected(loonPlot)
+#' loonPlot['showGuides'] <- TRUE
+#' plot(loonPlot)
+#'
+#' @export
+plot.loon <- function (x, y = NULL, ...) {
+    if (!is.null(y)) warning("argument y is ignored")
+    lg <- grid.loon(x, ...)
+    invisible(lg)
+}
+
+
 #' Create a grid grob from a loon widget handle
 #'
 #' Grid grobs are useful to create publication quality graphics.
@@ -184,37 +216,48 @@ cartesian2dGrob <- function(widget, interiorPlotGrob = NULL, name = NULL, gp = N
 
 
     if (!swapAxes) {
-        xlabelGrob <- textGrob(xlabel,
+        xlabelGrob <- condGrob(test = showLabels,
+                               grobFun = textGrob,
+                               name = "x label",
+                               label = xlabel,
                                y = unit(xylab_loc[1], "lines"),
                                gp = gpar(fontfamily = xlabelFont$family,
                                          fontsize = xlabelFont$size,
                                          fontface = xlabelFont$face
-                                         ),
-                               name = "x label")
-        ylabelGrob <- textGrob(ylabel,
+                               )
+        )
+        ylabelGrob <- condGrob(test = showLabels,
+                               grobFun = textGrob,
+                               name = "y label",
+                               label = ylabel,
                                x = unit(xylab_loc[2], "lines"),
                                rot = 90,
                                gp = gpar(fontfamily = ylabelFont$family,
                                          fontsize = ylabelFont$size,
                                          fontface = ylabelFont$face
-                                         ),
-                               name = "y label")
+                               )
+        )
     } else {
-        xlabelGrob <- textGrob(xlabel,
+        xlabelGrob <- condGrob(test = showLabels,
+                               grobFun = textGrob,
+                               name = "x label",
+                               label = xlabel,
                                x = unit(xylab_loc[2], "lines"),
                                rot = 90,
                                gp = gpar(fontfamily = xlabelFont$family,
                                          fontsize = xlabelFont$size,
                                          fontface = xlabelFont$face
-                               ),
-                               name = "x label")
-        ylabelGrob <- textGrob(ylabel,
+                               ))
+        ylabelGrob <- condGrob(test = showLabels,
+                               grobFun = textGrob,
+                               name = "y label",
+                               label = ylabel,
                                y = unit(xylab_loc[1], "lines"),
                                gp = gpar(fontfamily = ylabelFont$family,
                                          fontsize = ylabelFont$size,
                                          fontface = ylabelFont$face
-                               ),
-                               name = "y label")
+                               )
+                               )
     }
 
     axis <- loon.pretty(widget)
@@ -223,6 +266,17 @@ cartesian2dGrob <- function(widget, interiorPlotGrob = NULL, name = NULL, gp = N
     yaxis.major <- axis$yaxis.major
     yaxis.minor <- axis$yaxis.minor
 
+
+    titleGrob <- condGrob(test =  showLabels & (title != ""),
+                          grobFun = textGrob,
+                          name = "title",
+                          label = title,
+                          y = unit(1, "npc") + unit(.8, "lines"),
+                          gp = gpar(fontfamily = titleFont$family,
+                                    fontsize = titleFont$size,
+                                    fontface = titleFont$face
+                          ),
+                          vjust = .5)
     gTree(
         children = gList(
             rectGrob(gp = gpar(col = NA,
@@ -230,110 +284,108 @@ cartesian2dGrob <- function(widget, interiorPlotGrob = NULL, name = NULL, gp = N
                      name = "bounding box") ,
             gTree(
                 children = gList(
-                    if (showLabels) {
-                        gList(
-                            xlabelGrob,
-                            ylabelGrob,
-                            if (title != "") {
-                                textGrob(title,
-                                         name = "title",
-                                         y = unit(1, "npc") + unit(.8, "lines"),
-                                         gp = gpar(fontfamily = titleFont$family,
-                                                   fontsize = titleFont$size,
-                                                   fontface = titleFont$face
-                                         ),
-                                         vjust = .5)
-                            } else nullGrob(name = "null: no title") )
-                    } else nullGrob(name = "null: no labels"),
-                    if (showGuides){
-                        gTree(children = gList(
-
-                            rectGrob(gp = gpar(col = NA,
-                                               fill = as_hex6color(widget['guidesBackground'])
-                                               )
-                                     ),
-
-                            do.call(
-                                gList,
+                    gTree(children = gList(xlabelGrob,
+                                           ylabelGrob,
+                                           titleGrob),
+                          name = "labels"
+                    ),
+                    gTree(children = gList(
+                        # background
+                        condGrob(test = showGuides,
+                                 grobFun = rectGrob,
+                                 name = "guides background",
+                                 gp = gpar(col = NA,
+                                           fill = as_hex6color(widget['guidesBackground'])
+                                 )),
+                        # x major lines
+                        do.call(gList,
                                 lapply(xaxis.major,
                                        function(xaxis) {
-                                           linesGrob(x = unit(rep(xaxis,2 ), "native"),
-                                                     y =  unit(c(0, 1), "npc"),
-                                                     gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 2))
-
-                                       }
-                                )
-                            ),
-                            do.call(
-                                gList,
+                                           condGrob(test =  showGuides,
+                                                    grobFun = linesGrob,
+                                                    name = paste0("guidelines: xaxis (major), x = ", xaxis),
+                                                    x = unit(rep(xaxis, 2), "native"),
+                                                    y =  unit(c(0, 1), "npc"),
+                                                    gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 2))
+                                       })),
+                        # x minor lines
+                        do.call(gList,
                                 lapply(xaxis.minor,
                                        function(xaxis) {
-                                           linesGrob(x = unit(rep(xaxis,2 ), "native"),
-                                                     y =  unit(c(0, 1), "npc"),
-                                                     gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 1))
+                                           condGrob(test =  showGuides,
+                                                    grobFun = linesGrob,
+                                                    name = paste0("guidelines: xaxis (minor), x = ", xaxis),
+                                                    x = unit(rep(xaxis,2 ), "native"),
+                                                    y =  unit(c(0, 1), "npc"),
+                                                    gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 1))
 
-                                       }
-                                )
-                            ),
-                            do.call(
-                                gList,
+                                       })),
+                        # y major lines
+                        do.call(gList,
                                 lapply(yaxis.major,
                                        function(yaxis) {
-                                           linesGrob(x = unit(c(0, 1), "npc") ,
-                                                     y =  unit(rep(yaxis,2), "native"),
-                                                     gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 2))
+                                           condGrob(test =  showGuides,
+                                                    grobFun = linesGrob,
+                                                    name = paste0("guidelines: yaxis (major), y = ", yaxis),
+                                                    x = unit(c(0, 1), "npc") ,
+                                                    y =  unit(rep(yaxis,2), "native"),
+                                                    gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 2))
 
-                                       }
-                                )
-                            ),
-                            do.call(
-                                gList,
-                                lapply(yaxis.minor,
-                                       function(yaxis) {
-                                           linesGrob(x = unit(c(0, 1), "npc") ,
-                                                     y =  unit(rep(yaxis,2 ), "native"),
-                                                     gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 1))
+                                       })),
+                        # y minor lines
+                        do.call(
+                            gList,
+                            lapply(yaxis.minor,
+                                   function(yaxis) {
+                                       condGrob(test =  showGuides,
+                                                grobFun = linesGrob,
+                                                name = paste0("guidelines: yaxis (minor), y = ", yaxis),
+                                                x = unit(c(0, 1), "npc") ,
+                                                y =  unit(rep(yaxis,2 ), "native"),
+                                                gp = gpar(col = as_hex6color(widget['guidelines']), lwd = 1))
 
-                                       }
-                                )
+                                   }
                             )
-                        ),
-                        name = "guides")
-                    } else {
-                        nullGrob(name = "null: no guides")
-                        },
-
-                    if (showScales) {
-                        gList(
-                            xaxisGrob(
-                                at = xaxis.major,
-                                gp = gpar(fontfamily = scalesFont$family,
-                                          fontsize = scalesFont$size,
-                                          fontface = scalesFont$face
-                                ),
-                                name = "x axis"),
-                            yaxisGrob(
-                                at = yaxis.major,
-                                gp = gpar(fontfamily = scalesFont$family,
-                                          fontsize = scalesFont$size,
-                                          fontface = scalesFont$face
-                                ),
-                                name = "y axis")
                         )
-                    } else nullGrob(name = "null: no scales"),
+                    ),
+                    name = "guides"),
+
+                    # Axes
+                    gTree(children = gList(condGrob(test =  showScales,
+                                                    grobFun = xaxisGrob,
+                                                    name = "x axis",
+                                                    at = xaxis.major,
+                                                    gp = gpar(fontfamily = scalesFont$family,
+                                                              fontsize = scalesFont$size,
+                                                              fontface = scalesFont$face
+                                                    )),
+                                           condGrob(test =  showScales,
+                                                    grobFun = yaxisGrob,
+                                                    name = "y axis",
+                                                    at = yaxis.major,
+                                                    gp = gpar(fontfamily = scalesFont$family,
+                                                              fontsize = scalesFont$size,
+                                                              fontface = scalesFont$face
+                                                    ))),
+                          name = "axes"),
+
+
+                    # Clipping
                     clipGrob(name = "clipping region"),
+                    # Interior
                     interiorPlotGrob,
                     # draw boundary
-                    if(sum(margins) > 0) {
-                        rectGrob(name = "boundary rectangle",
-                                 gp=gpar(col = border, fill = NA, lwd=1))
-                    } else {
-                        nullGrob(name = "null: no boundary rectangle")
-                        }
+                    condGrob(test =  sum(margins) > 0,
+                             grobFun = rectGrob,
+                             name = "boundary rectangle",
+                             gp=gpar(col = border, fill = NA, lwd=1))
                 ),
                 vp = vpStack(
                     plotViewport(margins = margins, name = "plotViewport"),
-                    dataViewport(xscale = xlim, yscale = ylim, name = "dataViewport")
+                    dataViewport(xscale = xlim, yscale = ylim,
+                                 name = if (swapAxes)
+                                     "dataViewport: swapAxes" else
+                                         "dataViewport")
                 ),
                 name = "loon plot"
             )
@@ -349,13 +401,14 @@ loonGrob.l_layer_group <- function(target, name = NULL, gp = NULL, vp = NULL) {
 
     widget <- attr(target, "widget")
 
-    l_children_layers <- lapply(rev(l_layer_getChildren(target)), function(layerid) {
-        l_create_handle(c(widget, layerid))
-    })
+    l_children_layers <- lapply(rev(l_layer_getChildren(target)),
+                                function(layerid) {
+                                    l_create_handle(c(widget, layerid))
+                                })
 
-    l_visible_children_layer <- Filter(function(layerid) {
-        l_layer_isVisible(widget, layerid)
-    }, l_children_layers)
+    l_visible_children_layer <- Filter(
+        function(layerid) {l_layer_isVisible(widget, layerid)},
+        l_children_layers)
 
     l_children_grobs <- lapply(l_visible_children_layer, loonGrob)
 
@@ -1046,7 +1099,7 @@ getGridTextCoords  <-  function(text, angle, anchor, just){
     adjustedCoords
 }
 
-
+# TODO Export and fully document
 loon.pretty <- function(widget, digits = 3) {
 
     title <- widget['title']
@@ -1128,3 +1181,41 @@ loon.pretty <- function(widget, digits = 3) {
          yaxis.major = yaxis.major,
          yaxis.minor = yaxis.minor)
 }
+
+#' @title Create a named grob or a template grob depending on a test
+#'
+#' @description Creates and returns a grid object using the function
+#' given by `grobFun` when `test` is `TRUE`  Otherwise a simple `grob()`
+#' is produced with the same parameters.  All grob parameters are given in `...`.
+#'
+#' @param test Either `TRUE` or `FALSE` to indicate whether `grobFun` is to be used (default `TRUE`) or not.
+#' @param grobFun The function to be used to create the grob when `test = TRUE` (e.g. `textGrob`, `polygonGrob`, etc.).
+#' @param name The name to be used for the returned grob.
+#' @param ... The arguments to be given to the `grobFun` (or to `grob()` when `test = FALSE`).
+#'
+#' @return A grob as produced by either the `grobFun` given or by `grob()` using the remaining arguments.
+#' If `test = FALSE` then the name is suffixed by ": `grobFun name` arguments".
+#'
+#' @examples
+#' myGrob <- condGrob(test = (runif(1) > 0.5),
+#'                    grobFun = textGrob,
+#'                    name = "my label",
+#'                    label = "Some random text")
+#' myGrob
+#'
+#' @export
+
+condGrob <- function (test = TRUE,
+                      grobFun = grob,
+                      name = "grob name",
+                      ...){
+    if (test){
+        grobFun(name = name, ...)
+    } else {
+        grob(name = paste0(name,
+                           ": ",
+                           deparse(substitute(grobFun)),
+                           " arguments"),
+        ...)
+    }
+}
